import { EIP155_SIGNING_METHODS } from '@/data/EIP155Data'
import ModalStore from '@/store/ModalStore'
import SettingsStore from '@/store/SettingsStore'
import { web3wallet } from '@/utils/WalletConnectUtil'
import { SignClientTypes } from '@walletconnect/types'
import { Web3WalletTypes } from '@walletconnect/web3wallet'
import { useCallback, useEffect } from 'react'
import { useSnapshot } from 'valtio'

export default function useWalletConnectEventsManager() {
  const { web3WalletReady } = useSnapshot(SettingsStore.state)

  /******************************************************************************
   * 1. Open session proposal modal for confirmation / rejection
   *****************************************************************************/
  const onSessionProposal = useCallback((proposal: Web3WalletTypes.SessionProposal) => {
    ModalStore.open('SessionProposalModal', { proposal })
  }, [])

  /******************************************************************************
   * 3. Open request handling modal based on method that was used
   *****************************************************************************/
  const onSessionRequest = useCallback(async (requestEvent: Web3WalletTypes.SessionRequest) => {
    console.log('session_request', requestEvent)
    const { topic, params } = requestEvent
    const { request } = params
    const requestSession = web3wallet.getActiveSessions()[topic]

    switch (request.method) {
      case EIP155_SIGNING_METHODS.ETH_SIGN:
      case EIP155_SIGNING_METHODS.PERSONAL_SIGN:
        return ModalStore.open('SessionSignModal', { requestEvent, requestSession })

      case EIP155_SIGNING_METHODS.ETH_SIGN_TYPED_DATA:
      case EIP155_SIGNING_METHODS.ETH_SIGN_TYPED_DATA_V3:
      case EIP155_SIGNING_METHODS.ETH_SIGN_TYPED_DATA_V4:
        return ModalStore.open('SessionSignTypedDataModal', { requestEvent, requestSession })

      case EIP155_SIGNING_METHODS.ETH_SEND_TRANSACTION:
      case EIP155_SIGNING_METHODS.ETH_SIGN_TRANSACTION:
        return ModalStore.open('SessionSendTransactionModal', { requestEvent, requestSession })

      default:
        return ModalStore.open('SessionUnsuportedMethodModal', { requestEvent, requestSession })
    }
  }, [])

  /******************************************************************************
   * Set up WalletConnect event listeners
   *****************************************************************************/
  useEffect(() => {
    if (web3WalletReady) {
      web3wallet.on('session_proposal', onSessionProposal)
      web3wallet.on('session_request', onSessionRequest)
      web3wallet.on('session_delete', data => console.log('delete', data))
    }

    return () => {
      if (web3WalletReady) {
        web3wallet.off('session_proposal', onSessionProposal)
        web3wallet.off('session_request', onSessionRequest)
        web3wallet.off('session_delete', data => console.log('delete', data))
      }
    }
  }, [web3WalletReady, onSessionProposal, onSessionRequest])
}
